precision highp float;

#define ACX 8.0
#define ACY 24.0
#define TWO_PI 6.28318530718

varying vec2 st;

uniform sampler2D inputImageTexture;

uniform float seed;

uniform float offset;

const vec3 lightPos = vec3(1.0, -2.5, 1.6);
const vec3 viewPos = vec3(0.5, 2.0, 2.0);
const vec3 lightColor = vec3(1.0, 1.0, 1.0);
//uniform vec2 size;
mat2 rotateMatrix(float angle){
    return mat2(cos(angle), sin(angle),
    -sin(angle), cos(angle));
}


float rand(vec2 n) {
    return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float rand2(float x){
    return mod((mod(x, 13.0) + 1.0) * (mod(x, 123.0) + 1.0), 0.01) + 0.0025;
}

float noise(vec2 p){
    vec2 ip = floor(p);
    vec2 u = fract(p);
    u = u*u*(3.0-2.0*u);

    float res = mix(
    mix(rand(ip), rand(ip+vec2(1.0, 0.0)), u.x),
    mix(rand(ip+vec2(0.0, 1.0)), rand(ip+vec2(1.0, 1.0)), u.x), u.y);
    return res*res;
}

const mat2 mtx = mat2(0.80, 0.60, -0.60, 0.80);

float fbm(vec2 p)
{
    float f = 0.0;

    f += 0.500000*noise(p + 100.); p = mtx*p*2.02;
    f += 0.031250*noise(p); p = mtx*p*2.01;
    f += 0.250000*noise(p); p = mtx*p*2.03;
    f += 0.125000*noise(p); p = mtx*p*2.01;
    f += 0.062500*noise(p); p = mtx*p*2.04;
    f += 0.015625*noise(p + sin(100.));

    return f/0.96875;
}

float pattern(vec2 p)
{
    return fbm(p + fbm(p + fbm(p)));
}

//uniform float intensity;
void main() {

    vec2 uv = st;

    //    float aum = pattern(st);
    uv = vec2(uv.x, mod(uv.y + 1.0 - offset * 2.0, 1.0 * 2.0));
    uv.y = mix(uv.y, 1.0 * 2.0 - uv.y, step(1.0, uv.y));
    uv.y = 1.0 - uv.y;

    if(st.y > offset + 0.02){
        gl_FragColor = texture2D(inputImageTexture, st);
    }else {
        for (float i = 1.0; i < ACX; i++){
            vec2 random_center = vec2(rand(vec2(76.0, 67.0 + i * 3.2)) * 12.0 - 6.0,
            rand2(23.0 + i * 8.3) * 1200.0 - 6.0);
            float l = length(st - random_center);
            float n = 0.05 * sin(l * (80.0 + i * 3.6) + i + seed) * rand2(i * 13.14) * (1.35 - st.y);
            uv += rotateMatrix(TWO_PI / 4.0) * (st - random_center) * vec2(n) * exp(- l * 0.25);
        }


        for (float i = 1.0; i < ACY; i++){
            vec2 random_center = vec2(rand(vec2(21.0, 41.0 + i * 3.2)) * 4.0 - 2.0,
            rand2(13.0 + i * 5.3) * 800.0 - 13.8);
            float l = length(st - random_center);
            float n = 0.1 * sin(l * (80.0 + i * 8.0) + i + seed) * rand2(i * 24.14);
            uv +=  (st - random_center) * vec2(n) * exp(- l * 0.025);
        }

        //        uv.y = 1.0 - uv.y;
        vec2 invert = vec2(st.x, st.y);

        vec3 lightDir = normalize(lightPos - vec3(invert, 0.0));
        vec3 viewDir = normalize(viewPos - vec3(invert, 0.0));
        vec3 halfwayDir = normalize(viewDir + lightDir);

        vec3 normal = normalize(vec3(vec2(uv - invert), 1.0));
        float spec = pow(max(dot(normal, halfwayDir), 0.0), 32.0);
        vec3 specular = lightColor * spec;

        uv.y = min(max(uv.y, 0.0), 1.0);
        uv.x = min(max(uv.x, 0.0), 1.0);

        vec4 result = texture2D(inputImageTexture, vec2(uv.x, uv.y));

        result.rgb = result.rgb * 0.9 + specular * 0.3;

        vec4 col = texture2D(inputImageTexture, st);

        gl_FragColor = mix(col, result, smoothstep(1.0 - offset - 0.02, 1.0 - offset + 0.02, 1.0 - st.y));
    }

}