precision highp float;

#define MAX_CHROME_ERROR 0.2

#define MIN_CHROME_ERROR 0.02

varying vec2 st;
uniform sampler2D inputImageTexture;
uniform float selectHue;
uniform float targetHue;
uniform float minTolerance;
uniform float maxTolerance;

float rgb2gray(vec3 color){
    return dot(color, vec3(0.299, 0.587, 0.114));
}

vec3 rgb2hsv(vec3 color){
    float min_rgb = min(color.r, min(color.g, color.b));
    float v = max(color.r, max(color.g, color.b));
    float s = 0.;
    float h = 0.;
    if (v > min_rgb){
        s = (v - min_rgb) / v;
        h = 0.;
        if (v == color.r){
            h = (color.g - color.b) / (6. * (v - min_rgb));
        } else if (v == color.g){
            h = (color.b - color.r) / (6. * (v - min_rgb)) + 1. / 3.;
        } else if (v == color.b){
            h = (color.r - color.g) / (6. * (v - min_rgb)) + 2. / 3.;
        }
        if (h < 0.){
            h = h + 1.;
        }
    }
    return vec3(h, s, v);
}

vec3 hsv2rgb(vec3 c )
{
    vec3 rgb = clamp( abs(mod(c.x*6.0+vec3(0.0,4.0,2.0),6.0)-3.0)-1.0, 0.0, 1.0 );

    return c.z * mix( vec3(1.0), rgb, c.y);
}

void main() {
    vec4 color = texture2D(inputImageTexture, st);

    vec3 hsv = rgb2hsv(color.rgb);

    float curHue = hsv.x;

    if(abs(hsv.x - selectHue) > 0.5){
        curHue = mix(hsv.x + 1., hsv.x - 1., step(0.5, hsv.x));
    }

    float beta = 1.0 - smoothstep(minTolerance, maxTolerance, abs(hsv.x - selectHue));

    hsv.x = mod(hsv.x + (targetHue - selectHue) * beta, 1.0);

    color.rgb = hsv2rgb(hsv);

    gl_FragColor = color;
}