precision highp float;

varying vec2 st;
uniform sampler2D inputImageTexture;
uniform sampler2D portraitTexture;

//gimp
float rgb_max(vec3 rgb) {
    return max(rgb.r, max(rgb.g, rgb.b));
}

//gimp
float rgb_min(vec3 rgb) {
    return min(rgb.r, min(rgb.g, rgb.b));
}

//gimp
vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax-rgbmin;
        hsl.y = mix(delta/(2.-rgbmax-rgbmin), delta/(rgbmax+rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g-rgb.b)/delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b-rgb.r)/delta;
        else hsl.x = 4.+(rgb.r-rgb.g)/delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x+1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

//gimp
float hsl_value(float n1, float n2, float hue) {
    //    hue = mix(hue-6., mix(hue+6., hue, step(0., hue)), step(hue, 6.));
    if (hue < 0.){
        hue = hue + 6.;
    } else if (hue > 6.){
        hue = hue - 6.;
    }
    if (hue > 4.){
        return n1;
    } else if (hue > 3.){
        return n1 + (n2 - n1)*(4.-hue);
    } else if (hue > 1.){
        return n2;
    } else {
        return n1 + (n2 - n1) * hue;
    }
}

//gimp
vec3 hsl2rgb(vec3 hsl) {
    vec3 rgb = vec3(0.);
    if (hsl.y == 0.) {
        rgb.r = hsl.z;
        rgb.g = hsl.z;
        rgb.b = hsl.z;
    } else {
        float m2 = mix(hsl.z + hsl.y - hsl.z*hsl.y, hsl.z*(1.+hsl.y), step(hsl.z, .5));
        float m1 = 2.*hsl.z-m2;
        rgb.r = hsl_value(m1, m2, hsl.x * 6. + 2.);
        rgb.g = hsl_value(m1, m2, hsl.x * 6.);
        rgb.b = hsl_value(m1, m2, hsl.x * 6. - 2.);
    }
    return rgb;
}

float rgb2gray(vec3 c){
    return dot(c, vec3(0.299, 0.587, 0.114));
}

void main(){

    //origin
    vec3 origin = vec3(300./360., 375./360., 450./360.);

    vec4 color = texture2D(inputImageTexture, st);

    vec4 portrait = texture2D(portraitTexture, st);

    vec3 hsl = rgb2hsl(color.rgb);

    float h = mix(hsl.x + 1., hsl.x, step(origin.z - 1., hsl.x));

    float s = (rgb_max(color.rgb) - rgb_min(color.rgb)) / (rgb_max(color.rgb) + rgb_min(color.rgb) + 0.001);

    float a = portrait.a;

    if (h > origin.x && h < origin.z){
        float sat_s = max(0.4 - abs(s - 0.4), 0.) / 0.4;
        float hue_s = 1.0 - abs(h - origin.y) * 360. / 75.;
        a = min(sat_s * hue_s * a, 0.5);
    } else {
        a = 0.0;
    }

    gl_FragColor = vec4(a);
}