precision highp float;

varying vec2 st;
uniform sampler2D inputImageTexture;
uniform sampler2D portraitTexture;
uniform sampler2D lutTexture;
uniform float strength;

mat4 contrastMatrix( float contrast )
{
    float t = ( 1.0 - contrast ) / 2.0;

    return mat4( contrast, 0.0, 0.0, 0.0,
    0.0, contrast, 0.0, 0.0,
    0.0, 0, contrast, 0.0,
    t, t, t, 1.0 );

}



vec4 lut(vec4 color, float alpha){
    vec3 textureColor = color.rgb * 15. / 16. + vec3(1. / 32.);

    float blueColor = color.b * 63.0;
    vec2 quad1;
    quad1.y = floor(floor(blueColor) / 8.0);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);
    vec2 quad2;
    quad2.y = floor(ceil(blueColor) / 8.0);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);

    vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos1.y = (quad1.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);

    vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos2.y = (quad2.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);

    vec4 newColor = mix(texture2D(lutTexture, texPos1), texture2D(lutTexture, texPos2), fract(blueColor));

    newColor = contrastMatrix(16. / 15.) * newColor;
    return mix(color, vec4(newColor.rgb, color.w), alpha);
}


void main(){

    vec4 color = texture2D(inputImageTexture, st);

    float a = texture2D(portraitTexture, st).a;

    gl_FragColor = lut(color, min(a * strength, 1.0));
}